public class GameLogic {
  /**/
  private ComputerStatus cs;
  private Deck deck;
  private DeckReadoutPanel drop;
  private Hand ch, ph;
  private HandPanel chp, php;
  private HandStatusReadout hsro;
  private MainWindow mw;
  private ScoreReadout csro, psro;
  /**/
  public GameLogic ( MainWindow mw ) {
    /**/
    GameStatusPanel gsp;
    MainPanel mp;
    ScorePanel sp;
    /**/
    this.mw=mw;
    mp=mw.getMainPanel();
    sp=mp.getScorePanel();
    gsp=mp.getGameStatusPanel();
    /**/
    deck=mw.getDeck();
    chp=mw.getComputerHandPanel();
    php=mw.getPlayerHandPanel();
    csro=sp.getComputerReadout();
    psro=sp.getPlayerReadout();
    cs=mp.getComputerStatus();
    drop=mp.getDeckReadoutPanel();
    hsro=gsp.getHandStatusReadout();
    /**/
    drop.initGame();
    csro.initScore();
    psro.initScore();
  }
  /**/
  public void reset () {
    /**/
    chp.setHand(null);
    php.setHand(null);
    cs.reset();
    drop.initGame();
    csro.initScore();
    psro.initScore();
    /**/
    deck=new Deck();
    hsro.reset();
    startHand();
    /**/
    return;
  }
  /**/
  public void startHand () {
    /**/
    boolean computerBusted, playerBusted;
    int status;
    /**/
    enableHitStick();
    disableOk();
    hsro.reset();
    /**/
    if ( !dealHand() ) {
      gameOver();
      return;
    }
    else {
      for ( int i=0; i<4; ++i ) drop.decrement();
    }
    /**/
    if ( ch.getValue() > 21 ) computerBusted=true;
    else                      computerBusted=false;
    if ( ph.getValue() > 21 ) playerBusted=true;
    else                      playerBusted=false;
    /**/
    if ( computerBusted ) {
      if ( playerBusted ) {
        bothBusted();
        return;
      }
      else {
        computerBusted();
        return;
      }
    }
    else {
      if ( playerBusted ) {
        playerBusted();
        return;
      }
    }
    /**/
    status=computerDraws();
    if      ( status == 0 ) {
      gameOver();
      return;
    }
    else if ( status == 1 ) {
      computerBusted();
      return;
    }
  }
  /*
   *  option = 1 -> hit
   *  option = 2 -> stick
   *
   *  return: 0 = deck depleted
   *          1 = player busts
   *          2 = hand can continue
   *          3 = end hand
   */
  public int playerDraws ( int option ) {
    /**/
    Card c;
    /**/
    if ( option == 1 ) {
      if ( !deck.canDraw() ) {
        return 0;
      }
      else {
        c=deck.draw();
        drop.decrement();
        ph.add(c);
        php.setHand(ph);
        if ( ph.getValue() > 21 ) return 1;
        else                      return 2;
      }
    }
    else {
      return 3;
    }
  }
  /**/
  public void playerBusted () {
    /**/
    (ch.getCards())[0].flip();
    chp.setHand(ch);
    csro.incrementScore();
    hsro.displayStatus(2);
    disableHitStick();
    enableOk();
    /**/
    return;
  }
  /**/
  public void endHand () {
    /**/
    int chv, phv;
    boolean computerWins;
    Card chc, phc;
    /**/
    chv=ch.getValue();
    phv=ph.getValue();
    /**/
    if      ( chv == phv ) {
      chc=ch.getHighCard();
      phc=ph.getHighCard();
      if ( chc.lessThan(phc) ) computerWins=false;
      else                     computerWins=true;
    }
    else if ( chv <  phv ) computerWins=false;
    else                   computerWins=true;
    /**/
    (ch.getCards())[0].flip();
    chp.setHand(ch);
    /**/
    if ( computerWins ) {
      hsro.displayStatus(3);
      csro.incrementScore();
    }
    else {
      hsro.displayStatus(4);
      psro.incrementScore();
    }
    /**/
    disableHitStick();
    enableOk();
    /**/
    return;
  }
  /**/
  public void gameOver () {
    /**/
    int cs, ps;
    /**/
    cs=csro.getScore();
    ps=psro.getScore();
    if      ( cs == ps ) hsro.displayStatus(5);
    else if ( cs <  ps ) hsro.displayStatus(6);
    else                 hsro.displayStatus(7);
    /**/
    disableHitStick();
    /**/
    return;
  }
  /*
   *  return: true  = could complete the deal
   *          false = deck depleted
   */
  private boolean dealHand () {
    /**/
    Card [] cca, pca;
    /**/
    cca=new Card [2];
    pca=new Card [2];
    /**/
    if ( deck.canDraw() ) {
      cca[0]=deck.draw();
      cca[0].flip();
    }
    else {
      return false;
    }
    /**/
    if ( deck.canDraw() ) pca[0]=deck.draw();
    else return false;
    /**/
    if ( deck.canDraw() ) cca[1]=deck.draw();
    else return false;
    /**/
    if ( deck.canDraw() ) pca[1]=deck.draw();
    else return false;
    /**/
    ch=new Hand(cca);
    ph=new Hand(pca);
    chp.setHand(ch);
    php.setHand(ph);
    /**/
    return true;
  }
  /*
   *  return: 0 = deck depleted
   *          1 = computer is busted
   *          2 = hand can continue
   */
  private int computerDraws () {
    /**/
    int numDraws;
    double probabilityBust;
    Card c;
    /**/
    numDraws=0;
    while ( true ) {
      probabilityBust=probabilityBust();
      if ( probabilityBust <= 0.50 ) {
        if ( deck.canDraw() ) {
          c=deck.draw();
          ch.add(c);
          ++numDraws;
        }
        else {
          return 0;
        }
      }
      else {
        for ( int i=0; i<numDraws; ++i ) drop.decrement();
        cs.hits(numDraws);
        chp.setHand(ch);
        break;
      }
    }
    /**/
    if ( ch.getValue() > 21 ) return 1;
    else                      return 2;
  }
  /**/
  private double probabilityBust () {
    /**/
    int currentScore, numCards, count;
    Card [] ca;
    /**/
    currentScore=ch.getValue();
    ca=deck.getRemainingDeck();
    numCards=ca.length;
    count=0;
    for ( int i=0; i<numCards; ++i ) {
      if ( (ca[i].getValue()+currentScore) > 21 ) ++count;
    }
    /**/
    return (double)count/(double)numCards;
  }
  /**/
  private void bothBusted () {
    /**/
    cs.busts();
    (ch.getCards())[0].flip();
    chp.setHand(ch);
    hsro.displayStatus(0);
    disableHitStick();
    enableOk();
    /**/
    return;
  }
  /**/
  private void computerBusted () {
    /**/
    cs.busts();
    (ch.getCards())[0].flip();
    chp.setHand(ch);
    psro.incrementScore();
    hsro.displayStatus(1);
    disableHitStick();
    enableOk();
    /**/
    return;
  }
  /**/
  private void enableHitStick () {
    /**/
    ButtonPanel bp;
    /**/
    bp=mw.getMainPanel().getButtonPanel();
    bp.getHit().setEnabled(true);
    bp.getStick().setEnabled(true);
    /**/
    return;
  }
  /**/
  private void disableHitStick () {
    /**/
    ButtonPanel bp;
    /**/
    bp=mw.getMainPanel().getButtonPanel();
    bp.getHit().setEnabled(false);
    bp.getStick().setEnabled(false);
    /**/
    return;
  }
  /**/
  private void enableOk () {
    mw.getMainPanel().getGameStatusPanel().getOkButton().setEnabled(true);
  }
  /**/
  private void disableOk () {
    mw.getMainPanel().getGameStatusPanel().getOkButton().setEnabled(false);
  }
}
